/*************************************************************************************************
 * Portable scanner based on CC2500 single chip or related CC2500 module
 * Written by Valeriy Yatsenkov (aka Rover)
 * You may freely copy and redistribute this software if no fee is charged for use,
 * copying or distribution.  Such redistributions must retain the above copyright
 * notice.
 **************************************************************************************************
 * Precautions:
 * - Do appropriate current and voltage conversion between your microcontroller and CC2500 module.
 * - High voltage or High current may damage your CC2500 Module.
 **************************************************************************************************
 * scanner for 
 * base frequency 2400.009949 MHz (channel 0)
 * end frequency  2483.128540 MHz (channel 205)
 * channel spacing 405.456543 kHz
 **************************************************************************************************
 * Graphics implementation based on Arduino_GFX core library
 * https://github.com/adafruit/Adafruit-GFX-Library/archive/master.zip
 * 
 * carefully check what display chip used on your display module!!!
 *
 * library for ST7735 is Adafruit_ST7735
 * https://github.com/adafruit/Adafruit-ST7735-Library/archive/master.zip
 * library for Samsung S6D02A1 is Adafruit_QDTech
 * https://github.com/zigwart/Adafruit_QDTech
 *
 * Thanks Adafruit for great libraries, you can purchase display modules at http://www.adafruit.com
 */

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <SPI.h>
//#include <EEPROM.h>
#include "cc2500_REG.h"

#define TFT_CS     9      // display Select
#define SCAN_CS    10     // scanner Select
#define TFT_RST    7      // display Reset (0 if use Arduino reset)
#define TFT_DC     8      // display Data/Command control line

#define RSSI_TRESHOLD 100  // treshold of RSSI alarm level
#define RSSI_OFFSET   95   // offset for displayed data
#define RSSI_OVERSAMPLE 50 // number of measures for each channel to found a max RSSI value
#define MAX_CHAN_QTY  205  // max number of channel for spacing 405.456543MHz
#define MAX_DISP_LINE 159  // max horizontal display resolution
#define MAX_SAMPLING 10    // qty of samples in each iteration (1...100)

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST); // use hardware SPI as more fast

byte cal[MAX_CHAN_QTY], rssi_data[MAX_DISP_LINE], data;
byte RSSI_data;
int RSSI_dbm, RSSI_max;

void setup(void) {
  //Serial.begin(9600);

  pinMode(TFT_CS, OUTPUT);
  pinMode(SCAN_CS,OUTPUT);
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV2);  // max SPI speed, 1/2 F_CLOCK
  digitalWrite(TFT_CS,HIGH);
  digitalWrite(SCAN_CS,HIGH);
  init_CC2500();  // initialise CC2500 registers

  tft.initR(INITR_BLACKTAB);     // initialize a ST7735S chip, black tab
  tft.setRotation(1);            // landscape mode (or 1 for rotation 180deg)
  tft.fillScreen(ST7735_BLACK);  // clear display to black
  tft.setCursor(0,5);
  tft.setTextColor(ST7735_WHITE);
  tft.print("Initialize scanner...");

  // calibration procedure
  // collect and save calibration data for each displayed channel
  for (int i=0; i<MAX_DISP_LINE; i++)
  {
    WriteReg(CHANNR, i);		// set channel
    WriteReg(SIDLE, 0x3D);		// idle mode
    WriteReg(SCAL, 0x3D);		// start manual calibration
    delayMicroseconds(800);		// wait for calibration
    data = ReadReg(FSCAL1);             // read calibration value
    cal[i] = data;			// and store it
    tft.drawFastVLine(i, 20, 10, ST7735_BLUE); //progress bar on screen
  }

  tft.fillScreen(ST7735_BLACK);         // clear display to black

  WriteReg(CHANNR, 0x00);		// set channel
  WriteReg(SFSTXON, 0x3D);	        // calibrate and wait
  delayMicroseconds(800);               // settling time, refer to datasheet
  WriteReg(SRX, 0x3D);			// enable rx
}

void loop() {
  for (int i=0; i<=MAX_DISP_LINE; i++)
  {
    WriteReg(CHANNR, i);		// set channel
    WriteReg(FSCAL1, cal[i]);		// restore calibration value for this channel
    delayMicroseconds(300);	        // settling time, refer to datasheet

    RSSI_max = -120;

    for (int j=0; j<=MAX_SAMPLING; j++) {// collect samples for max value
      digitalWrite(SCAN_CS,LOW);        // select the scanner chip
      SPI.transfer(REG_RSSI);	        // read RSSI register
      RSSI_data = SPI.transfer(0);
      digitalWrite(SCAN_CS,HIGH);

      // convert RSSI data from 2's complement to signed decimal
      if(RSSI_data >= 128) {
        RSSI_dbm = (RSSI_data-256)/2-70;
      }
      else {
        RSSI_dbm = RSSI_data/2-70;
      }    
      if (RSSI_dbm > RSSI_max) RSSI_max = RSSI_dbm; // keep maximum   
    }

    RSSI_max += RSSI_OFFSET;
    if(RSSI_max > 110) RSSI_max = 110;
    if(RSSI_max < 0) RSSI_max = 0;
    rssi_data[i] = RSSI_max;
    //tft.drawFastVLine(i, 17, 110-RSSI_max, ST7735_BLACK);
    //tft.drawFastVLine(i, 127-RSSI_max, RSSI_max, ST7735_GREEN);
  }
  DrawScreen();
}

void DrawScreen() {
  for(int x=1; x<=MAX_DISP_LINE; x++) {
    tft.drawFastVLine(x, 60, 67-rssi_data[x], ST7735_BLACK);
    tft.drawFastVLine(x, 127-rssi_data[x], rssi_data[x], ST7735_GREEN);
  }
}

void init_CC2500(){
  WriteReg(SRES,0x3D);       // software reset for CC2500
  WriteReg(FSCTRL1,0x0F);    // Frequency Synthesizer Control
  WriteReg(PKTCTRL0,0x12);   // Packet Automation Control
  WriteReg(FREQ2,0x5C);      // Frequency control word, high byte
  WriteReg(FREQ1,0x4E);      // Frequency control word, middle byte
  WriteReg(FREQ0,0xDE);      // Frequency control word, low byte
  WriteReg(MDMCFG4,0x0D);    // Modem Configuration
  WriteReg(MDMCFG3,0x3B);    // Modem Configuration
  WriteReg(MDMCFG2,0x00);    // Modem Configuration 0x30 - OOK modulation, 0x00 - FSK modulation (better sensitivity)
  WriteReg(MDMCFG1,0x23);    // Modem Configuration
  WriteReg(MDMCFG0,0xFF);    // Modem Configuration
  WriteReg(MCSM1,0x0F);      // Always stay in RX mode
  WriteReg(MCSM0,0x04);      // Main Radio Control State Machine Configuration
  WriteReg(FOCCFG,0x15);     // Frequency Offset Compensation configuration
  WriteReg(AGCCTRL2,0x83);   // AGC Control
  WriteReg(AGCCTRL1,0x00);   // AGC Control
  WriteReg(AGCCTRL0,0x91);   // AGC Control
  WriteReg(FSCAL3,0xEA);     // Frequency Synthesizer Calibration
  WriteReg(FSCAL2,0x0A);     // Frequency Synthesizer Calibration 
  WriteReg(FSCAL1,0x00);     // Frequency Synthesizer Calibration 
  WriteReg(FSCAL0,0x11);     // Frequency Synthesizer Calibration
}

void WriteReg(char addr, char value){
  digitalWrite(SCAN_CS,LOW);
  while (digitalRead(MISO) == HIGH) {
  };
  SPI.transfer(addr);
  SPI.transfer(value);
  digitalWrite(SCAN_CS,HIGH);
}

char ReadReg(char addr){
  addr = addr + 0x80;
  digitalWrite(SCAN_CS,LOW);
  while (digitalRead(MISO) == HIGH) {
  };
  SPI.transfer(addr);
  char y = SPI.transfer(0);
  digitalWrite(SCAN_CS,HIGH);
  return y;  
}






















